unit ApeUrnListFrame;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "ApeUrnListFrame.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, ComCtrls, ToolWin, ImgList, ActnList, StdCtrls, VTCtrls,
  XDOM_2_4;

type
  TUrnListFrame = class(TFrame)
    ActionList1: TActionList;
    UpAction: TAction;
    DownAction: TAction;
    AddAction: TAction;
    DeleteAction: TAction;
    DisabledButtonsImageList: TImageList;
    EnabledButtonsImageList: TImageList;
    ToolBar: TToolBar;
    FullTitleUpToolButton: TToolButton;
    FullTitleDownToolButton: TToolButton;
    FullTitleAddToolButton: TToolButton;
    FullTitleDeleteToolButton: TToolButton;
    VirtualWideStringList: TVirtualWideStringList;
    EditToolButton: TToolButton;
    EditAction: TAction;
    procedure AddActionExecute(Sender: TObject);
    procedure EditActionUpdate(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure VirtualWideStringListNewText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
    procedure DeleteActionUpdate(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure UpActionUpdate(Sender: TObject);
    procedure DownActionUpdate(Sender: TObject);
    procedure UpActionExecute(Sender: TObject);
    procedure DownActionExecute(Sender: TObject);
    procedure FrameExit(Sender: TObject);
  private
    function GetLines: TdomWideStringList;
  public
    property Lines: TdomWideStringList read GetLines;
  end;

type
  TApeUrnListFrame = class(TUrnListFrame);

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeUrnListFrame]);
end;

{$R *.dfm}

{ TUrnListFrame }

function TUrnListFrame.GetLines: TdomWideStringList;
begin
  result:= VirtualWideStringList.Lines;
end;

procedure TUrnListFrame.AddActionExecute(Sender: TObject);
var
  tn,next_tn: PVirtualNode;
begin
  with VirtualWideStringList do begin
    if IsEditing then EndEditNode;
    tn:= FocusedNode;
    if assigned(tn) then begin
      Lines.insert(succ(tn.Index),'');
      next_tn:= tn.nextSibling;
      if assigned(next_tn) then begin
        FocusedNode:= next_tn;
        Selected[FocusedNode]:= True;
      end;
    end else begin
      Lines.Add('');
      FocusedNode := GetFirst;
      Selected[FocusedNode]:= True;
    end; {if ... else ...}
  end; {with ...}
  EditActionExecute(Sender);
end;

procedure TUrnListFrame.EditActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= assigned(VirtualWideStringList.FocusedNode);
end;

procedure TUrnListFrame.EditActionExecute(Sender: TObject);
var
  tn: PVirtualNode;
begin
  with VirtualWideStringList do begin
    tn:= FocusedNode;
    if assigned(tn) then
      EditNode(tn,-1);
  end; {with ...}
end;

procedure TUrnListFrame.VirtualWideStringListNewText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  NewText: WideString);
begin
  if Node.Index < Cardinal(VirtualWideStringList.Lines.Count)
    then VirtualWideStringList.Lines[Node.Index] := NewText;
end;

procedure TUrnListFrame.DeleteActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= assigned(VirtualWideStringList.FocusedNode);
end;

procedure TUrnListFrame.DeleteActionExecute(Sender: TObject);
var
  tn: PVirtualNode;
  l: Cardinal;
begin
  with VirtualWideStringList do begin
    if IsEditing then EndEditNode;
    tn:= FocusedNode;
    if assigned(tn) then begin
      l := pred(Cardinal(Lines.Count));
      if tn.Index < l then begin
        Lines.Delete(tn.Index);
      end else if tn.Index = l then begin
        Lines.Delete(tn.Index);
        FocusedNode := GetLast(nil);
        Selected[FocusedNode]:= True;
      end;
    end; {if ...}
  end; {with ...}
end;

procedure TUrnListFrame.UpActionExecute(Sender: TObject);
var
  tn,previous_tn: PVirtualNode;
begin
  with VirtualWideStringList do begin
    if IsEditing then EndEditNode;
    tn:= FocusedNode;
    if assigned(tn) then begin
      previous_tn:= tn.PrevSibling;
      if assigned(previous_tn) then begin
        Lines.exchange(tn.Index,pred(tn.Index));
        FocusedNode:= previous_tn;
        Selected[FocusedNode]:= True;
      end;
    end; {if ...}
  end; {with ...}
end;

procedure TUrnListFrame.UpActionUpdate(Sender: TObject);
var
  tn: PVirtualNode;
begin
  tn:= VirtualWideStringList.FocusedNode;
  if assigned(tn) then begin
    if assigned(tn.prevSibling)
      then (Sender as TAction).Enabled:= true
      else (Sender as TAction).Enabled:= false;
  end else (Sender as TAction).Enabled:= false;
end;

procedure TUrnListFrame.DownActionExecute(Sender: TObject);
var
  tn,next_tn: PVirtualNode;
begin
  with VirtualWideStringList do begin
    if IsEditing then EndEditNode;
    tn:= FocusedNode;
    if assigned(tn) then begin
      next_tn:= tn.nextSibling;
      if assigned(next_tn) then begin
        Lines.exchange(tn.Index,succ(tn.Index));
        FocusedNode:= next_tn;
        Selected[FocusedNode]:= True;
      end;
    end; {if ...}
  end; {with ...}
end;

procedure TUrnListFrame.DownActionUpdate(Sender: TObject);
var
  tn: PVirtualNode;
begin
  tn:= VirtualWideStringList.FocusedNode;
  if assigned(tn) then begin
    if assigned(tn.nextSibling)
      then (Sender as TAction).Enabled:= true
      else (Sender as TAction).Enabled:= false;
  end else (Sender as TAction).Enabled:= false;
end;

procedure TUrnListFrame.FrameExit(Sender: TObject);
begin
  with VirtualWideStringList do 
    if IsEditing then EndEditNode;
end;

end.
