unit CrossReferenceDisplayFrame;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "CrossReferenceDisplayFrame.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which 
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this 
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  Xdom_2_4,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Buttons, ExtCtrls, ActnList, ImgList, ToolWin,
  ApeCommentDisp, ResourceDisplayFrame, ApeHelpDisp;

type

  TCrossRefDispFrame = class(TFrame)
    PageControl1: TPageControl;
    CommentTabSheet: TTabSheet;
    ApparatusTabSheet: TTabSheet;
    ApeCommentDisplay1: TApeCommentDisplay;
    HelpTabSheet: TTabSheet;
    ApeHelpDisplay: TApeHelpDisplay;
    ApparatusDisplay: TApeResourceDisplay;
    procedure PageControl1Change(Sender: TObject);
  protected
    FCommentText: wideString;
    FLastAppUrn: wideString;
    function GetApparatusFontName: TFontName; virtual;
    function GetDefaultFile: TFilename; virtual;
    function GetDefaultResource: TFilename;
    procedure SetApparatusFontName(const Value: TFontName); virtual;
    procedure SetDefaultFile(const Value: TFilename); virtual;
    procedure SetDefaultResource(const Value: TFilename);
  public
    procedure AddHelp(const path: TFilename;
                      const labelText: wideString); virtual;
    procedure DisplayComment(const CommentText: wideString); virtual;
    procedure InitApparatus(const Input: TApeDisplayPacket); virtual;
    procedure ShowApparatusTab; virtual;
    procedure ShowCommentTab; virtual;
    procedure ShowHelpTab; virtual;
    function SwitchToHelp(const  Index: Integer): boolean; virtual;
    procedure UpdateComment; virtual;
    procedure UpdateHelp; virtual;
  published
    property ApparatusFontName: TFontName read GetApparatusFontName write SetApparatusFontName;
    property DefaultFile: TFilename read GetDefaultFile write SetDefaultFile;
    property DefaultResource: TFilename read GetDefaultResource write SetDefaultResource;
  end;

  TApeCrossReferenceDisplay = class(TCrossRefDispFrame);

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [ TApeCrossReferenceDisplay ]);
end;


{$R *.DFM}

{ TCrossRefDispFrame }

procedure TCrossRefDispFrame.AddHelp(const path: TFilename;
                                     const labelText: wideString);
begin
  ApeHelpDisplay.Add(path,labelText);
  HelpTabSheet.TabVisible:= true;
end;

procedure TCrossRefDispFrame.DisplayComment(const CommentText: wideString);
begin
  FCommentText:= CommentText;
  UpdateComment;
end;

function TCrossRefDispFrame.GetApparatusFontName: TFontName;
begin
  Result:= ApparatusDisplay.Font.Name;
end;

function TCrossRefDispFrame.GetDefaultFile: TFilename;
begin
  Result:= ApeCommentDisplay1.DefaultFile;
end;

function TCrossRefDispFrame.GetDefaultResource: TFilename;
begin
  Result:= ApparatusDisplay.DefaultResource;
end;

procedure TCrossRefDispFrame.InitApparatus(const Input: TApeDisplayPacket);
begin
  ApparatusDisplay.Init(Input);
end;

procedure TCrossRefDispFrame.PageControl1Change(Sender: TObject);
begin
// xxx ???
(*  if PageControl1.ActivePage = ApparatusTabSheet then
    if not (csDesigning in ComponentState)  // suppress URL loading at design time
      then DisplayApparatus(FLastAppUrn);*)
end;

procedure TCrossRefDispFrame.SetApparatusFontName(const Value: TFontName);
begin
  ApparatusDisplay.Font.Name:= Value;
end;

procedure TCrossRefDispFrame.SetDefaultFile(const Value: TFilename);
begin
  ApeCommentDisplay1.DefaultFile:= Value;
end;

procedure TCrossRefDispFrame.SetDefaultResource(const Value: TFilename);
begin
  ApparatusDisplay.DefaultResource:= Value;
end;

procedure TCrossRefDispFrame.ShowApparatusTab;
begin
  PageControl1.ActivePage:= ApparatusTabSheet;
end;

procedure TCrossRefDispFrame.ShowCommentTab;
begin
  PageControl1.ActivePage:= CommentTabSheet;
end;

procedure TCrossRefDispFrame.ShowHelpTab;
begin
  PageControl1.ActivePage:= HelpTabSheet;
end;

function TCrossRefDispFrame.SwitchToHelp(const Index: Integer): boolean;
begin
  result:= ApeHelpDisplay.Display(Index);
  if result then PageControl1.ActivePage:= HelpTabSheet;
end;

procedure TCrossRefDispFrame.UpdateComment;
begin
  ApeCommentDisplay1.DisplayText(FCommentText,ApparatusFontName);
end;

procedure TCrossRefDispFrame.UpdateHelp;
begin
  ApeHelpDisplay.Home;
end;

end.
