unit LanguageAndTextVSTFrame;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "LanguageAndTextVSTFrame.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which 
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, ComCtrls, ToolWin, ImgList, ActnList, ExtCtrls, StdCtrls;

type

  TLanguageAndTextTreeData = record
    domElement: TdomElement;
  end;

  PLanguageAndTextTreeData = ^TLanguageAndTextTreeData;

  TLangAndTextVSTFrame = class(TFrame)
    ToolBar1: TToolBar;
    FullTitleUpToolButton: TToolButton;
    FullTitleDownToolButton: TToolButton;
    FullTitleAddToolButton: TToolButton;
    FullTitleDeleteToolButton: TToolButton;
    VirtualStringTree: TVirtualStringTree;
    EnabledButtonsImageList: TImageList;
    DisabledButtonsImageList: TImageList;
    ActionList1: TActionList;
    UpAction: TAction;
    DownAction: TAction;
    AddAction: TAction;
    DeleteAction: TAction;
    FSplitter: TSplitter;
    TextMemo: TMemo;
    procedure AddActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure DeleteActionUpdate(Sender: TObject);
    procedure DownActionExecute(Sender: TObject);
    procedure DownActionUpdate(Sender: TObject);
    procedure UpActionExecute(Sender: TObject);
    procedure UpActionUpdate(Sender: TObject);
    procedure VirtualStringTreeFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure VirtualStringTreeFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VirtualStringTreeGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var Text: WideString);
    procedure VirtualStringTreeNewText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; Text: WideString);
    procedure VirtualStringTreeGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure VirtualStringTreeNodeMoved(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VirtualStringTreeExit(Sender: TObject);
    procedure TextMemoChange(Sender: TObject);
    procedure ActionList1Update(Action: TBasicAction;
      var Handled: Boolean);
    procedure VirtualStringTreeEditing(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean);
  private
    FContainerElement: TdomElement;
    FCopyLanguageOnAddition: boolean;
    FDefaultLanguage: wideString;
    FStandardNamespaceURI: wideString;
    FStandardQualifiedName: wideString;
    FAdditionalAttrValue: wideString;
    FAdditionalAttrQualifiedName: wideString;
    FAdditionalAttrNamespaceURI: wideString;
    function GetSplitterAutoSnap: boolean;
    function GetSplitterBeveled: boolean;
    function GetSplitterHeight: integer;
    function GetSplitterMinSize: NaturalNumber;
    function GetSplitterResizeStyle: TResizeStyle;
    function GetSplitterTop: integer;
    procedure SetSplitterAutoSnap(const Value: boolean);
    procedure SetSplitterBeveled(const Value: boolean);
    procedure SetSplitterHeight(const Value: integer);
    procedure SetSplitterMinSize(const Value: NaturalNumber);
    procedure SetSplitterResizeStyle(const Value: TResizeStyle);
    procedure SetSplitterTop(const Value: integer);
  protected
    function DoStrToWideStrConversion(S: string): wideString; virtual;
    function DoWideStrToStrConversion(S: wideString): string; virtual;
    function GetAdditionalAttrLocalName: wideString; virtual;
    function GetAdditionalAttrPrefix: wideString; virtual;
    function GetStandardLocalName: wideString; virtual;
    function GetStandardPrefix: wideString; virtual;
    procedure InitializeTreeView; virtual;
    procedure UpdateMemoText(Sender: TBaseVirtualTree;
                             Node: PVirtualNode); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddChildElement(const element: TdomElement);
    procedure InitializeDisplay(const ContainerEl: TdomElement;
                                const stdNamespaceURI,
                                      stdQualifiedName,
                                      addAttrNamespaceURI,
                                      addAttrQualifiedName,
                                      addAttrValue: wideString);
    property AdditionalAttrLocalName: wideString read GetAdditionalAttrLocalName;
    property AdditionalAttrNamespaceURI: wideString read FAdditionalAttrNamespaceURI;
    property AdditionalAttrPrefix: wideString read GetAdditionalAttrPrefix;
    property AdditionalAttrQualifiedName: wideString read FAdditionalAttrQualifiedName;
    property AdditionalAttrValue: wideString read FAdditionalAttrValue;
    property ContainerElement: TdomElement read FContainerElement;
    property StandardLocalName: wideString read GetStandardLocalName;
    property StandardNamespaceURI: wideString read FStandardNamespaceURI;
    property StandardPrefix: wideString read GetStandardPrefix;
    property StandardQualifiedName: wideString read FStandardQualifiedName;
  published
    property CopyLanguageOnAddition: boolean read FCopyLanguageOnAddition write FCopyLanguageOnAddition default false;
    property DefaultLanguage: wideString read FDefaultLanguage write FDefaultLanguage;
    property SplitterAutoSnap: boolean read GetSplitterAutoSnap write SetSplitterAutoSnap;
    property SplitterBeveled: boolean read GetSplitterBeveled write SetSplitterBeveled;
    property SplitterHeight: integer read GetSplitterHeight write SetSplitterHeight;
    property SplitterMinSize: NaturalNumber read GetSplitterMinSize write SetSplitterMinSize;
    property SplitterResizeStyle: TResizeStyle read GetSplitterResizeStyle write SetSplitterResizeStyle;
    property SplitterTop: integer read GetSplitterTop write SetSplitterTop;
  end;

  TApeLanguageAndTextEditor = class(TLangAndTextVSTFrame);

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeLanguageAndTextEditor]);
end;

const
  xmlns_xml: wideString = 'http://www.w3.org/XML/1998/namespace';

procedure setLanguage(const Element: TdomElement; const Value: wideString);
var
  attr: TdomAttr;
begin
  with Element do
    if Value = '' then begin
      if hasAttributeNS(xmlns_xml,'lang') then begin
        attr:= removeAttributeNS(xmlns_xml,'lang');
        ownerDocument.freeAllNodes(TdomNode(attr));
      end;
    end else setAttributeNS(xmlns_xml,'xml:lang',Value);
end;

{$R *.dfm}

constructor TLangAndTextVSTFrame.Create(AOwner: TComponent);
begin
  inherited;
  FAdditionalAttrValue:= '';
  FAdditionalAttrQualifiedName:= '';
  FAdditionalAttrNamespaceURI:= '';
  FContainerElement:= nil;
  FCopyLanguageOnAddition:= false;
  FDefaultLanguage:= '';
  FStandardNamespaceURI:= '';
  FStandardQualifiedName:= '';
end;

function TLangAndTextVSTFrame.GetSplitterAutoSnap: boolean;
begin
  result:= FSplitter.AutoSnap;
end;

function TLangAndTextVSTFrame.GetSplitterBeveled: boolean;
begin
  result:= FSplitter.Beveled;
end;

function TLangAndTextVSTFrame.GetSplitterHeight: integer;
begin
  result:= FSplitter.Height;
end;

function TLangAndTextVSTFrame.GetSplitterMinSize: NaturalNumber;
begin
  result:= FSplitter.MinSize;
end;

function TLangAndTextVSTFrame.GetSplitterResizeStyle: TResizeStyle;
begin
  result:= FSplitter.ResizeStyle;
end;

function TLangAndTextVSTFrame.GetSplitterTop: integer;
begin
  result:= FSplitter.Top;
end;

procedure TLangAndTextVSTFrame.SetSplitterAutoSnap(const Value: boolean);
begin
  FSplitter.AutoSnap:= Value;
end;

procedure TLangAndTextVSTFrame.SetSplitterBeveled(const Value: boolean);
begin
  FSplitter.Beveled:= Value;
end;

procedure TLangAndTextVSTFrame.SetSplitterHeight(const Value: integer);
begin
  FSplitter.Height:= Value;
end;

procedure TLangAndTextVSTFrame.SetSplitterMinSize(
  const Value: NaturalNumber);
begin
  FSplitter.MinSize:= Value;
end;

procedure TLangAndTextVSTFrame.SetSplitterResizeStyle(
  const Value: TResizeStyle);
begin
  FSplitter.ResizeStyle:= Value;
end;

procedure TLangAndTextVSTFrame.SetSplitterTop(const Value: integer);
var
  deltaV: integer;
begin
  deltaV:= FSplitter.Top - Value;
  TextMemo.Height:= TextMemo.Height + deltaV;
end;

function TLangAndTextVSTFrame.DoStrToWideStrConversion(
  S: string): wideString;
begin
  result:= strToPcdata(trim(S),GetACPEncodingType);
end;

function TLangAndTextVSTFrame.DoWideStrToStrConversion(
  S: wideString): string;
begin
  result:= trim(pcdataToStr(S,GetACPEncodingType));
end;

function TLangAndTextVSTFrame.GetAdditionalAttrLocalName: wideString;
begin
  if FAdditionalAttrQualifiedName <> ''
    then result:= XMLExtractLocalName(FAdditionalAttrQualifiedName)
    else result:= '';
end;

function TLangAndTextVSTFrame.GetAdditionalAttrPrefix: wideString;
begin
  if FAdditionalAttrQualifiedName <> ''
    then result:= XMLExtractPrefix(FAdditionalAttrQualifiedName)
    else result:= '';
end;

function TLangAndTextVSTFrame.GetStandardLocalName: wideString;
begin
  if FStandardQualifiedName <> ''
    then result:= XMLExtractLocalName(FStandardQualifiedName)
    else result:= '';
end;

function TLangAndTextVSTFrame.GetStandardPrefix: wideString;
begin
  if FStandardQualifiedName <> ''
    then result:= XMLExtractPrefix(FStandardQualifiedName)
    else result:= '';
end;

procedure TLangAndTextVSTFrame.AddChildElement(const element: TdomElement);
var
  tn: PVirtualNode;
  data: PLanguageAndTextTreeData;
begin
  with VirtualStringTree do begin
    tn:= AddChild(nil);
    data:= GetNodeData(tn);
    data.domElement:= element;
  end;
end;

procedure TLangAndTextVSTFrame.InitializeDisplay(
  const ContainerEl: TdomElement; const stdNamespaceURI, stdQualifiedName,
  addAttrNamespaceURI, addAttrQualifiedName, addAttrValue: wideString);
var
  locName,prfx: wideString;
begin
  if (stdNamespaceURI <> '') or (stdQualifiedName <> '') then begin
    if not IsXmlName(stdQualifiedName)
      then raise EInvalid_Character_Err.create('Invalid character error.');
    try
      locName:= XMLExtractLocalName(stdQualifiedName);
      prfx:= XMLExtractPrefix(stdQualifiedName);
    except
      raise ENamespace_Err.create('Namespace error.');
    end;
    if (stdNamespaceURI = '') and (prfx <> '')
      then raise ENamespace_Err.create('Namespace error.');
    if (prfx = 'xml') and (stdNamespaceURI <> 'http://www.w3.org/XML/1998/namespace')
      then raise ENamespace_Err.create('Namespace error.');
  end;

  if (addAttrNamespaceURI <> '') or (addAttrQualifiedName <> '') or (addAttrValue <> '')then begin
    if not IsXmlName(addAttrQualifiedName)
      then raise EInvalid_Character_Err.create('Invalid character error.');
    try
      locName:= XMLExtractLocalName(addAttrQualifiedName);
      prfx:= XMLExtractPrefix(addAttrQualifiedName);
    except
      raise ENamespace_Err.create('Namespace error.');
    end;
    if ( ((prfx = 'xmlns') or (addAttrQualifiedName = 'xmlns'))
      and not (addAttrNamespaceURI ='http://www.w3.org/2000/xmlns/') )
        then raise ENamespace_Err.create('Namespace error.');
    if (addAttrNamespaceURI = '') and (prfx <> '')
      then raise ENamespace_Err.create('Namespace error.');
    if (prfx = 'xml') and (addAttrNamespaceURI <> 'http://www.w3.org/XML/1998/namespace')
      then raise ENamespace_Err.create('Namespace error.');
  end;

  FContainerElement:= ContainerEl;

  FStandardNamespaceURI:= stdNamespaceURI;
  FStandardQualifiedName:= stdQualifiedName;
  FAdditionalAttrNamespaceURI:= addAttrNamespaceURI;
  FAdditionalAttrQualifiedName:= addAttrQualifiedName;
  FAdditionalAttrValue:= addAttrValue;

  InitializeTreeView;
end;

procedure TLangAndTextVSTFrame.InitializeTreeView;
var
  childEl: TdomElement;
  additionalAttr: TdomAttr;
begin
  with VirtualStringTree do begin
    BeginUpdate;
    try

      Clear;
      with Header do begin
        Options:= Options + [hoVisible];
        with Columns.Items[0] do
          Options:= Options + [coVisible];
        AutoSizeIndex:= 1;
      end;

      if assigned(FContainerElement) then begin
        // Traverse child elements
        childEl:= FContainerElement.getFirstChildElementNS(FStandardNamespaceURI,StandardLocalName);
        while assigned(childEl) do begin
          if FAdditionalAttrQualifiedName = '' then begin
            // No additional attribute to consider
            AddChildElement(childEl);
          end else begin
            // Additional attribute to consider
            additionalAttr:= childEl.getAttributeNodeNS(FAdditionalAttrNamespaceURI,AdditionalAttrLocalName);
            if assigned(additionalAttr) then begin
              if additionalAttr.value = FAdditionalAttrValue
                then AddChildElement(childEl);
            end; {if assigned(ape_type) ...}
          end;
          childEl:= childEl.getNextSiblingElementNS(FStandardNamespaceURI,StandardLocalName);
        end; {while ...}
      end; {if ...}

    finally
      EndUpdate;
    end; {try ... }
  end; {with VirtualStringTree ...}
end;

procedure TLangAndTextVSTFrame.ActionList1Update(Action: TBasicAction;
  var Handled: Boolean);
var
  tn: PVirtualNode;
begin
  tn:= VirtualStringTree.FocusedNode;
  if assigned(tn)
    then TextMemo.Enabled:= true
    else TextMemo.Enabled:= false;
end;

procedure TLangAndTextVSTFrame.UpdateMemoText(Sender: TBaseVirtualTree;
 Node: PVirtualNode);
var
  domElement: TdomElement;
  Data: PLanguageAndTextTreeData;
begin
  data:= Sender.GetNodeData(Node);
  domElement:= data.domElement;
  if assigned(domElement)
    then with domElement do
     TextMemo.Text:= DoWideStrToStrConversion(textContent);
end;

procedure TLangAndTextVSTFrame.DeleteActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= assigned(VirtualStringTree.FocusedNode);
end;

procedure TLangAndTextVSTFrame.UpActionUpdate(Sender: TObject);
var
  tn: PVirtualNode;
begin
  tn:= VirtualStringTree.FocusedNode;
  if assigned(tn) then begin
    if assigned(tn.prevSibling)
      then (Sender as TAction).Enabled:= true
      else (Sender as TAction).Enabled:= false;
  end else (Sender as TAction).Enabled:= false;
end;

procedure TLangAndTextVSTFrame.DownActionUpdate(Sender: TObject);
var
  tn: PVirtualNode;
begin
  tn:= VirtualStringTree.FocusedNode;
  if assigned(tn) then begin
    if assigned(tn.nextSibling)
      then (Sender as TAction).Enabled:= true
      else (Sender as TAction).Enabled:= false;
  end else (Sender as TAction).Enabled:= false;
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeNewText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  Text: WideString);
var
  data: PLanguageAndTextTreeData;
begin
  with Sender do begin
    data:= GetNodeData(Node);
    case column of
      0: setLanguage(data.domElement,Text);
      1: begin
        with data.domElement do begin
          clear;
          appendChild(ownerDocument.createTextNode(Text));
          UpdateMemoText(Sender,Node);
        end; {with ...}
      end;
    end; {case ...}
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeFocusChanged(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex);
begin
  // Note: the test whether a node can really be edited is done in the OnEditing event
  if assigned(Node) then begin
    UpdateMemoText(Sender,Node);
    Sender.EditNode(Node,Column);
  end; {if assigned(Node) ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeGetNodeDataSize(
  Sender: TBaseVirtualTree; var NodeDataSize: Integer);
begin
  NodeDataSize := SizeOf(TLanguageAndTextTreeData);
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeGetText(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var Text: WideString);
var
  domElement: TdomElement;
  xml_lang: TdomAttr;
  Data: PLanguageAndTextTreeData;
begin
  data:= Sender.GetNodeData(Node);
  domElement:= data.domElement;
  sender.Font.Color:= clBlack;
  if assigned(domElement) then with domElement do begin
    case column of
      0: begin
        xml_lang:= getAttributeNodeNS(xmlns_xml,'lang');
        if assigned(xml_lang)
          then text:= xml_lang.value
          else text:= '';
      end;
      1: text:= textContent;
    end;
  end; {if ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeFreeNode(
  Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  oldElement: TdomNode;
  data: PLanguageAndTextTreeData;
begin
  with VirtualStringTree do begin
    data:= GetNodeData(Node);
    oldElement:= data.domElement;
    with oldElement do begin
      if assigned(parentNode)
        then parentNode.removeChild(oldElement);
      ownerDocument.freeAllNodes(oldElement);
    end; {with ...}
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeNodeMoved(
  Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  movedElement,targetElement: TdomNode;
  data: PLanguageAndTextTreeData;
  next_tn: PVirtualNode;
begin
  with VirtualStringTree do begin
    data:= GetNodeData(Node);
    movedElement:= data.domElement;
    with movedElement do begin
      if assigned(parentNode)
        then parentNode.removeChild(movedElement);
      next_tn:= Node.NextSibling;
      if assigned(next_tn) then begin
        data:= GetNodeData(next_tn);
        targetElement:= data.domElement;
        ContainerElement.insertBefore(movedElement,targetElement);
      end else ContainerElement.appendChild(movedElement);
    end; {with ...}
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.AddActionExecute(Sender: TObject);
var
  tn,lastFocused: PVirtualNode;
  newData,focusedData: PLanguageAndTextTreeData;
  newElement, oldElement: TdomElement;
begin
  if (StandardQualifiedName = '') or not assigned(ContainerElement) then exit;
  with VirtualStringTree do begin
    newElement:= ContainerElement.ownerDocument.createElementNS(StandardNamespaceUri,StandardQualifiedName);
    try
      if AdditionalAttrQualifiedName <> ''
        then newElement.setAttributeNS(AdditionalAttrNamespaceURI,AdditionalAttrQualifiedName,AdditionalAttrValue);
      lastFocused:= FocusedNode;
      EndEditNode;
      if assigned(lastFocused) then begin
        tn:= InsertNode(lastFocused,amInsertAfter);
        focusedData:= GetNodeData(lastFocused);
        oldElement:= focusedData.domElement;
        ContainerElement.insertBefore(newElement,oldElement.nextSibling);
      end else begin
        tn:= InsertNode(nil,amAddChildLast);
        ContainerElement.appendChild(newElement);
        oldElement:= nil;
      end;
      newData:= GetNodeData(tn);
      newData.domElement:= newElement;
      if FCopyLanguageOnAddition then begin
        if assigned(oldElement)
          then setLanguage(newElement,oldElement.getAttributeNSNormalizedValue(xmlns_xml,'lang'))
          else setLanguage(newElement,FDefaultLanguage);
        FocusedColumn:= 1;
      end else begin
        setLanguage(newElement,FDefaultLanguage);
        FocusedColumn:= 0;
      end; {if FCopyLanguageOnAddition ... else ...}
      FocusedNode:= tn;
      Selected[lastFocused]:= False;
      Selected[tn]:= True;
    except
      if assigned(newElement.ParentNode)
        then newElement.parentNode.removeChild(newElement);
      ContainerElement.ownerDocument.freeAllNodes(TdomNode(newElement));
      raise;
    end; {try ...}
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.DeleteActionExecute(Sender: TObject);
var
  fn,tn: PVirtualNode;
begin
  with VirtualStringTree do begin
    tn:= FocusedNode;
    fn:= tn.PrevSibling;
    if not assigned(fn)
      then fn:= tn.NextSibling;
    if assigned(tn) then begin
      FocusedNode:= nil;
      DeleteNode(tn);
    end; {if ...}
    FocusedNode:= fn;
    Selected[fn]:= True;
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.UpActionExecute(Sender: TObject);
var
  tn,previous_tn: PVirtualNode;
begin
  with VirtualStringTree do begin
    tn:= FocusedNode;
    if assigned(tn) then begin
      previous_tn:= tn.PrevSibling;
      if assigned(previous_tn)
        then MoveTo(tn,previous_tn,amInsertBefore,false);
    end; {if ...}
    FocusedNode:= tn;
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.DownActionExecute(Sender: TObject);
var
  tn,next_tn: PVirtualNode;
begin
  with VirtualStringTree do begin
    tn:= FocusedNode;
    if assigned(tn) then begin
      next_tn:= tn.nextSibling;
      if assigned(next_tn)
        then MoveTo(tn,next_tn,amInsertAfter,false);
    end; {if ...}
    FocusedNode:= tn;
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeExit(Sender: TObject);
begin
  VirtualStringTree.EndEditNode;
end;

procedure TLangAndTextVSTFrame.TextMemoChange(Sender: TObject);
var
  data: PLanguageAndTextTreeData;
begin
  with VirtualStringTree do begin
    data:= GetNodeData(FocusedNode);
    with data.domElement do begin
      clear;
      appendChild(ownerDocument.createTextNode(DoStrToWideStrConversion((Sender as TMemo).Text)));
    end; {with ...}
    InvalidateNode(FocusedNode);
  end; {with ...}
end;

procedure TLangAndTextVSTFrame.VirtualStringTreeEditing(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  var Allowed: Boolean);
begin
  if Column = 1 then Allowed := false;  // Do not edit the text column.
end;

end.
